#include <Denoiser.hpp>
#include <gaussian/kernel_cpp/fuyGauss.hpp>

/*!
 * \class DenoiserGaussian
 *
 * \brief Restored a frame affected by gaussian noise
 *
 * This class restores a frame affected by gaussian noise
 *
 */
class DenoiserGaussian: public Denoiser {
public:
  /**
   *
   * @param height frame height
   * @param width frame h
   * @param alfa_ functional parameter alfa
   * @param beta_ functional parameter beta
   * @param fixed_cycles flag for variable/fixed number of cycles
   * @param max_cycles maximum number of cycles
   * @param trace_time_ tells if time should be measured
   */
  DenoiserGaussian(unsigned int height, unsigned int width, float alfa_, float beta_, bool fixed_cycles, unsigned int max_cycles, bool trace_time_) :
      Denoiser(height, width, fixed_cycles, max_cycles, trace_time), alfa(alfa_), beta(beta_) {
  }
  /**
   *
   * @param in input frame
   * @param x pixel index
   * @param noisymap noisy pixels map
   * @return
   */
  unsigned char restorePixel(unsigned char *in, unsigned int x, int *noisymap) {
    return fuy(in, x, noisymap, width, height, alfa, beta);
  }

private:
  float alfa, beta;

};
