#ifndef _OCV_VIDEO_OUTPUT_HPP_
#define _OCV_VIDEO_OUTPUT_HPP_

#include <ocv/cast.hpp>
#include <Output.hpp>

#include <opencv2/opencv.hpp>
#include <string>

/*!
 * \class VideoOutputOCV
 *
 * \brief Writes frames to a video stream using OpenCV
 *
 * This class writes frames to a video stream using OpenCV.
 */
class VideoOutputOCV: public Output {
public:

  VideoOutputOCV(string out_fname, int height, int width, double fps) :
      Output(height, width) {
    //get frame info
    int fourcc = cv::VideoWriter::fourcc('X', 'V', 'I', 'D'); // static_cast<int>(capture.get(CV_CAP_PROP_FOURCC));
    //out_fname.append(in_fname.substr(in_fname.length() - 3, in_fname.length())); //same extension as input file
    out_fname.append(".avi");
    outputVideo.open(out_fname, fourcc, fps, cv::Size((int) width, (int) height), false);

    if (!outputVideo.isOpened()) {
      std::cout << "!!! Output video could not be opened" << out_fname << std::endl;
    }
  }

  void writeFrame(unsigned char *frame) {
    //write
    outputVideo << cv::Mat(height, width, CV_8U, frame, cv::Mat::AUTO_STEP); // convert uc* to Mat
  }

private:
  cv::VideoWriter outputVideo;
};
#endif
