#ifndef _DENOISERSPD_HPP_
#define _DENOISERSPD_HPP_

#include <Denoiser.hpp>
#include <spd/kernel_cpp/fuy.hpp>
#include <spd/kernel_cpp/pow_table.hpp>

/*!
 * \class DenoiserSPD
 *
 * \brief Restored a frame affected by salt-and-pepper noise
 *
 * This class restores a frame affected by salt-and-pepper noisy
 * by applying the modified Chan variational method.
 */
class DenoiserSPD: public Denoiser {
public:
  DenoiserSPD(unsigned int height, unsigned int width, float alfa_, float beta_, bool fixed_cycles, unsigned int max_cycles, bool trace_time_) :
      Denoiser(height, width, fixed_cycles, max_cycles, trace_time_), alfa(alfa_), beta(beta_), pt(new pow_table(alfa_)) {
  }

  ~DenoiserSPD() {
    delete pt;
  }

  /*!
   * \brief Restores a pixel
   *
   * It applies the modified Chan kernel over a pixel.
   *
   * \param in is the input frame
   * \param x is the index of the pixel to be restored
   * \param noisymap is the noisy-map
   *
   * \return the value of the restored pixel
   */
  unsigned char restorePixel(unsigned char *in, unsigned int x, int *noisymap) {
    return fuy(in, x, noisymap, width, height, alfa, beta, *pt);
  }

private:
  float alfa, beta;
  pow_table *pt;

};
#endif
