#!/bin/bash
#
# Author: Dalvan Griebler
# Email: dalvangriebler@gmail.com
#	plzip script for plotting
#


if [ "$#" -lt 3 ]
then
    echo -e " Error: Please, enter correct arguments as follows:\n $0 <log_dir> <workers_limit> <problem_size>";
    exit 1;
fi

#constant variables
LOG_EXTENSION=".log"; #in case you may change your extension with the following command (from .dat to .log) `for f in *.dat; do mv -- "$f" "${f%.dat}.log"; done`
GNP_DATA_EXTENSION=".dat";
PLOT_DIR="plot";
DAT_DIR="dat";
GNP_DIR="gnp";
LOG_DIR=$1;

mkdir -p $PLOT_DIR;
mkdir -p $PLOT_DIR/$DAT_DIR;
mkdir -p $PLOT_DIR/$GNP_DIR;

declare -a app;
declare -a kernel;

kernel=("denoiser");

#all tests
#app=("spar" "spar-ond" "spar-blk" "spar-ond-blk" "spar-ord" "spar-ord-ond" "spar-ord-blk" "spar-ord-ond-blk" "tbb" "ff");
app=("spar-ord" "spar-ord-ond" "spar-ord-pqueue" "spar-ord-pqueue-ond");

declare -a filter;
declare -a filcol;
declare -a metrics;

filter=("Time" "Memory" "cache-miss" "Voluntary_ctx_switches" "NonVoluntary_ctx_switches" "energy-pkg" "energy-ram");
filcol=("2" "6" "5" "4" "4" "5" "5");
metrics=("Seconds" "KB" "KB" "Frequency" "Frequency" "Joules" "Joules");

#filter=("Wall" "Memory" "cache-miss" "Voluntary_ctx_switches" "NonVoluntary_ctx_switches" "energy-pkg" "energy-ram");
#filcol=("3" "6" "5" "4" "4" "6" "6");

#filter=("Wall" "cache-miss");
#filcol=("3" "5");

#exact position where is the time filter to compute HPC metrics only
HPC_F=0; 

app_len=${#app[@]};
filter_len=${#filter[@]};
kernel_len=${#kernel[@]};

#filtering all logs
for((f=0;f<${filter_len};f++)) do
	for((k=0;k<${kernel_len};k++)) do
		if [ ${filter[$f]} == "Voluntary_ctx_switches" ]
		then
			grep ${filter[$f]} $LOG_DIR/${kernel[$k]}"-serial"$LOG_EXTENSION |grep -v ${filter[$f+1]} |awk -v col=${filcol[$f]} '{ print $col }' >  $LOG_DIR/${kernel[$k]}"-serial-"${filter[$f]}$GNP_DATA_EXTENSION; 
		else 
			grep ${filter[$f]} $LOG_DIR/${kernel[$k]}"-serial"$LOG_EXTENSION |awk -v col=${filcol[$f]} '{ print $col }' >  $LOG_DIR/${kernel[$k]}"-serial-"${filter[$f]}$GNP_DATA_EXTENSION;
		fi
		for((a=0;a<${app_len};a++)) do
			for((w=1; w<=$2; w++)) do
				if [ ${filter[$f]} == "Voluntary_ctx_switches" ]
				then
					grep ${filter[$f]} $LOG_DIR/${kernel[$k]}"-"${app[$a]}$w$LOG_EXTENSION |grep -v ${filter[$f+1]} |awk -v col=${filcol[$f]} '{ print $col }' > $LOG_DIR/${kernel[$k]}"-"${app[$a]}$w"-"${filter[$f]}$GNP_DATA_EXTENSION;
				else
					grep ${filter[$f]} $LOG_DIR/${kernel[$k]}"-"${app[$a]}$w$LOG_EXTENSION |awk -v col=${filcol[$f]} '{ print $col }' > $LOG_DIR/${kernel[$k]}"-"${app[$a]}$w"-"${filter[$f]}$GNP_DATA_EXTENSION;
				fi
			done;
		done;
	done;
done;

#plotting execution

#calculating means and stdv
for((f=0;f<${filter_len};f++)) do
	for((k=0;k<${kernel_len};k++)) do
		for((a=0;a<${app_len};a++)) do
			awk '{sum += $1; y+=$1^2} END {printf 0; printf "\t"; printf sum/NR; printf "\t"; print sqrt(y/NR-(sum/NR)^2)}' $LOG_DIR/${kernel[$k]}"-serial-"${filter[$f]}$GNP_DATA_EXTENSION >  $LOG_DIR/${kernel[$k]}"-"${app[$a]}"-"${filter[$f]}"-means"$GNP_DATA_EXTENSION;
			for((w=1; w<=$2; w++)) do
				awk '{sum += $1; y+=$1^2} END {printf '$w'; printf "\t"; printf sum/NR; printf "\t"; print sqrt(y/NR-(sum/NR)^2)}' $LOG_DIR/${kernel[$k]}"-"${app[$a]}$w"-"${filter[$f]}$GNP_DATA_EXTENSION >>  $LOG_DIR/${kernel[$k]}"-"${app[$a]}"-"${filter[$f]}"-means"$GNP_DATA_EXTENSION;
			done;
		done;
	done;
done;

#calculating speed_up and efficiency
#for((f=0;f<${filter_len};f++)) do
	for((k=0;k<${kernel_len};k++)) do
		for((a=0;a<${app_len};a++)) do
			awk 'BEGIN {s_t; p_t;} {i++; if(i==1){s_t=$2; printf ($1)"\t"(s_t/s_t)"\t"; print (s_t/s_t)/($1+1); }else{p_t=$2; printf ($1)"\t"(s_t/p_t)"\t"; print (s_t/p_t)/(i-1); };} END{} ' $LOG_DIR/${kernel[$k]}"-"${app[$a]}"-"${filter[$HPC_F]}"-means"$GNP_DATA_EXTENSION > $LOG_DIR/${kernel[$k]}"-"${app[$a]}"-"${filter[$HPC_F]}"-hpc-perf"$GNP_DATA_EXTENSION;
		done;
	done;
#done;

#calculating throughput and latency(milliseconds) 
#for((f=0;f<${filter_len};f++)) do
	for((k=0;k<${kernel_len};k++)) do
		for((a=0;a<${app_len};a++)) do
			awk 'BEGIN {t; p='$3';} {i++; if(i==1){t=$2; printf ($1)"\t"(p/t)"\t"; print (t/p)*1000; }else{t=$2; printf ($1)"\t"(p/t)"\t"; print (t/p)*1000; };} END{} '  $LOG_DIR/${kernel[$k]}"-"${app[$a]}"-"${filter[$HPC_F]}"-means"$GNP_DATA_EXTENSION > $LOG_DIR/${kernel[$k]}"-"${app[$a]}"-"${filter[$HPC_F]}"-stream-perf"$GNP_DATA_EXTENSION;
		done;
	done;
#done;

#moving all filtered data
mv $LOG_DIR/*".dat" $PLOT_DIR/$DAT_DIR/

DATE=`date`;
FONT_SIZE="15";
FONT_TYPE="Helvetica";

LINE_FORMTS="
set xlabel \"Number of Replicas\"

set style line 1 lt 1 lc rgb \"#62BCFF\" lw LW				#cyan
set style line 2 lt 2 lc rgb \"#0BA825\" lw LW				#green
set style line 3 lt 3 lc rgb \"#0368FF\" lw LW				#blue
set style line 4 lt 4 lc rgb \"#CB1B00\" lw LW				#red
set style line 5 lt 5 lc rgb \"#000000\" lw LW				#black
set style line 6 lt 6 lc rgb \"#002C64\" lw LW				#blue_dark
set style line 7 lt 7 lc rgb \"#5B9AD1\" lw LW				#blue_light
set style line 8 lt 8 lc rgb \"#A9A9A9\" lw LW				#dark gray
set style line 9 lt 9 lc rgb \"#8A2BE2\" lw LW				#blue violet
set style line 10 lt 10 lc rgb \"#00CED1\" lw LW			#dark turquoise
"

#plotting all means
for((f=0;f<${filter_len};f++)) do
for((k=0;k<${kernel_len};k++)) do
HEADER_PLOT_MEANS="#Author: Dalvan Griebler
#Email: dalvangriebler@gmail.com
#Version: $DATE
set encoding iso_8859_1
set terminal postscript eps solid color font '"$FONT_TYPE","$FONT_SIZE"'
set output '"${kernel[$k]}"-"${filter[$f]}"-means.eps'
set style data dots
set grid
set boxwidth 0.1
LW=2
"$LINE_FORMTS"
set logscale y 2
set key outside top horizontal nobox font '"$FONT_TYPE","$FONT_SIZE"'
set title '"${kernel[$k]}" ("${filter[$f]}")' offset -2,0,1
"
HEADER_PLOT_MEANS+="set ylabel \"${metrics[$f]}\" offset 2,1,1 
"
printf "$HEADER_PLOT_MEANS" > $PLOT_DIR/$GNP_DIR/${kernel[$k]}"-"${filter[$f]}"-means.gnp";

PLOTING_STR="plot ";
for((a=0;a<${app_len};a++)) do
	if [ $a == 0 ]
	then
		PLOTING_STR+="'../"$DAT_DIR"/"${kernel[$k]}"-"${app[$a]}"-"${filter[$f]}"-means"$GNP_DATA_EXTENSION"' using 1:2:xtic(1) with linespoints ls "$(($a+1))" title \""${app[$a]}"\" axes x1y1,'' using :2:3 with errorbars ls "$(($a+1))" notitle\\";
	else
		PLOTING_STR+="
	,'../"$DAT_DIR"/"${kernel[$k]}"-"${app[$a]}"-"${filter[$f]}"-means"$GNP_DATA_EXTENSION"' using 1:2:xtic(1) with linespoints ls "$(($a+1))" title \""${app[$a]}"\" axes x1y1,'' using :2:3 with errorbars ls "$(($a+1))" notitle\\";
	fi
done;
printf "$PLOTING_STR" >> $PLOT_DIR/$GNP_DIR/${kernel[$k]}"-"${filter[$f]}"-means.gnp";
printf "
" >> $PLOT_DIR/$GNP_DIR/${kernel[$k]}"-"${filter[$f]}"-means.gnp";
done;
done;


#----------------------------------------------------
#plotting hpc-pref (speedup)
#for((f=0;f<${filter_len};f++)) do
for((k=0;k<${kernel_len};k++)) do
HEADER_PLOT_HPC="#Author: Dalvan Griebler
#Email: dalvangriebler@gmail.com
#Version: $DATE
set encoding iso_8859_1
set terminal postscript eps solid color font '"$FONT_TYPE","$FONT_SIZE"'
set output '"${kernel[$k]}"-"${filter[$HPC_F]}"-hpc-speedup.eps'
set style data dots
set grid
set boxwidth 0.1
LW=2
"$LINE_FORMTS"
#set logscale y 2
set key outside top horizontal nobox font '"$FONT_TYPE","$FONT_SIZE"'
set title '"${kernel[$k]}" (HPC Performance)' offset -2,0,1
set ylabel \"Speed-up\" offset 2,1,1
";

printf "$HEADER_PLOT_HPC" > $PLOT_DIR/$GNP_DIR/${kernel[$k]}"-hpc-speedup.gnp";
PLOTING_STR="plot ";
for((a=0;a<${app_len};a++)) do
	if [ $a == 0 ]
	then
		PLOTING_STR+="'../"$DAT_DIR"/"${kernel[$k]}"-"${app[$a]}"-"${filter[$HPC_F]}"-hpc-perf"$GNP_DATA_EXTENSION"' using 1:2:xtic(1) with linespoints ls "$(($a+1))" title \""${app[$a]}"\" axes x1y1\\";
	else
		PLOTING_STR+="
	,'../"$DAT_DIR"/"${kernel[$k]}"-"${app[$a]}"-"${filter[$HPC_F]}"-hpc-perf"$GNP_DATA_EXTENSION"' using 1:2:xtic(1) with linespoints ls "$(($a+1))" title \""${app[$a]}"\" axes x1y1\\";
	fi
done;
printf "$PLOTING_STR" >> $PLOT_DIR/$GNP_DIR/${kernel[$k]}"-hpc-speedup.gnp";
printf "
" >> $PLOT_DIR/$GNP_DIR/${kernel[$k]}"-hpc-speedup.gnp";
done;
#done;



#plotting hpc-pref (Efficiency) 
#for((f=0;f<${filter_len};f++)) do
for((k=0;k<${kernel_len};k++)) do
HEADER_PLOT_HPC="#Author: Dalvan Griebler
#Email: dalvangriebler@gmail.com
#Version: $DATE
set encoding iso_8859_1
set terminal postscript eps solid color font '"$FONT_TYPE","$FONT_SIZE"'
set output '"${kernel[$k]}"-"${filter[$HPC_F]}"-hpc-efficiency.eps'
set style data dots
set grid
set boxwidth 0.1
LW=2
"$LINE_FORMTS"
unset logscale
set key outside top horizontal nobox font '"$FONT_TYPE","$FONT_SIZE"'
set title '"${kernel[$k]}" (HPC Performance)' offset -2,0,1
set ylabel \"Efficiency\" offset 2,1,1
";

printf "$HEADER_PLOT_HPC" > $PLOT_DIR/$GNP_DIR/${kernel[$k]}"-hpc-efficiency.gnp";
PLOTING_STR="plot ";
for((a=0;a<${app_len};a++)) do
	if [ $a == 0 ]
	then
		PLOTING_STR+="'../"$DAT_DIR"/"${kernel[$k]}"-"${app[$a]}"-"${filter[$HPC_F]}"-hpc-perf"$GNP_DATA_EXTENSION"' using 1:3:xtic(1) with linespoints ls "$(($a+1))" title \""${app[$a]}"\" axes x1y1\\";
	else
		PLOTING_STR+="
	,'../"$DAT_DIR"/"${kernel[$k]}"-"${app[$a]}"-"${filter[$HPC_F]}"-hpc-perf"$GNP_DATA_EXTENSION"' using 1:3:xtic(1) with linespoints ls "$(($a+1))" title \""${app[$a]}"\" axes x1y1\\";
	fi
done;
printf "$PLOTING_STR" >> $PLOT_DIR/$GNP_DIR/${kernel[$k]}"-hpc-efficiency.gnp";
printf "
" >> $PLOT_DIR/$GNP_DIR/${kernel[$k]}"-hpc-efficiency.gnp";
done;
#done;



#plotting stream-pref (Throughput)
#for((f=0;f<${filter_len};f++)) do
for((k=0;k<${kernel_len};k++)) do
HEADER_PLOT_HPC="#Author: Dalvan Griebler
#Email: dalvangriebler@gmail.com
#Version: $DATE
set encoding iso_8859_1
set terminal postscript eps solid color font '"$FONT_TYPE","$FONT_SIZE"'
set output '"${kernel[$k]}"-"${filter[$HPC_F]}"-stream-throughput.eps'
set style data dots
set grid
set boxwidth 0.1
LW=2
"$LINE_FORMTS"
unset logscale
set key outside top horizontal nobox font '"$FONT_TYPE","$FONT_SIZE"'
set title '"${kernel[$k]}" (Stream Performance)' offset -2,0,1
set ylabel \"Throughput of "$3"\" offset 2,1,1
";

printf "$HEADER_PLOT_HPC" > $PLOT_DIR/$GNP_DIR/${kernel[$k]}"-stream-throughput.gnp";
PLOTING_STR="plot ";
for((a=0;a<${app_len};a++)) do
	if [ $a == 0 ]
	then
		PLOTING_STR+="'../"$DAT_DIR"/"${kernel[$k]}"-"${app[$a]}"-"${filter[$HPC_F]}"-stream-perf"$GNP_DATA_EXTENSION"' using 1:2:xtic(1) with linespoints ls "$(($a+1))" title \""${app[$a]}"\" axes x1y1\\";
	else
		PLOTING_STR+="
	,'../"$DAT_DIR"/"${kernel[$k]}"-"${app[$a]}"-"${filter[$HPC_F]}"-stream-perf"$GNP_DATA_EXTENSION"' using 1:2:xtic(1) with linespoints ls "$(($a+1))" title \""${app[$a]}"\" axes x1y1\\";
	fi
done;
printf "$PLOTING_STR" >> $PLOT_DIR/$GNP_DIR/${kernel[$k]}"-stream-throughput.gnp";
printf "
" >> $PLOT_DIR/$GNP_DIR/${kernel[$k]}"-stream-throughput.gnp";
done;
#done;

#plotting stream-pref (Latency)
#for((f=0;f<${filter_len};f++)) do
for((k=0;k<${kernel_len};k++)) do
HEADER_PLOT_HPC="#Author: Dalvan Griebler
#Email: dalvangriebler@gmail.com
#Version: $DATE
set encoding iso_8859_1
set terminal postscript eps solid color font '"$FONT_TYPE","$FONT_SIZE"'
set output '"${kernel[$k]}"-"${filter[$HPC_F]}"-stream-latency.eps'
set style data dots
set grid
set boxwidth 0.1
LW=2
"$LINE_FORMTS"
unset logscale
set key outside top horizontal nobox font '"$FONT_TYPE","$FONT_SIZE"'
set title '"${kernel[$k]}" (Stream Performance)' offset -2,0,1
set ylabel \"Latency(milliseconds)\" offset 2,1,1
";

printf "$HEADER_PLOT_HPC" > $PLOT_DIR/$GNP_DIR/${kernel[$k]}"-stream-latency.gnp";
PLOTING_STR="plot ";
for((a=0;a<${app_len};a++)) do
	if [ $a == 0 ]
	then
		PLOTING_STR+="'../"$DAT_DIR"/"${kernel[$k]}"-"${app[$a]}"-"${filter[$HPC_F]}"-stream-perf"$GNP_DATA_EXTENSION"' using 1:3:xtic(1) with linespoints ls "$(($a+1))" title \""${app[$a]}"\" axes x1y1\\";
	else
		PLOTING_STR+="
	,'../"$DAT_DIR"/"${kernel[$k]}"-"${app[$a]}"-"${filter[$HPC_F]}"-stream-perf"$GNP_DATA_EXTENSION"' using 1:3:xtic(1) with linespoints ls "$(($a+1))" title \""${app[$a]}"\" axes x1y1\\";
	fi
done;
printf "$PLOTING_STR" >> $PLOT_DIR/$GNP_DIR/${kernel[$k]}"-stream-latency.gnp";
printf "
" >> $PLOT_DIR/$GNP_DIR/${kernel[$k]}"-stream-latency.gnp";
done;
#done;

#generating all graphs 
cd $PLOT_DIR/$GNP_DIR
`gnuplot *.gnp`;

#converting eps to pdf
`find . -name "*.eps" -exec epstopdf {} ";"`

#removing eps
`rm -rf *.eps`

